/*
 * ===========================
 * VDK Visual Development Kit
 * odbc subsystem
 * Version 0.1
 * Revision 0.0
 * January 2001
 * ===========================
 *
 * Copyright (C) 2001, Mario Motta
 * Developed by Pierre-Louis Malatray <pierrelouis.malatray@free.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#ifndef _vdkodbccolumn_h
#define _vdkodbccolumn_h

#include <vdkodbc/vdkodbc.h>
#include <vdkodbc/vdkodbcbase.h>

/*!
\class VDKODBCColumn
\brief Contains one column characteristics

This object will be owned by a table object
*/

class VDKODBCColumn : public VDKODBCBase
{
	private:
		//! Data Type identifier
		int			DataType;
		//! Decimal digits
		int			DecDigits;
		//! Precision
		int			Radix;
		//! Nullable
		int			Nullable;
		//! SQL data type
		int			SqlDataType;
		//! DateTime Sub type
		int			DateTimeSub;
		//! Column size
		int			Size;
		//! Buffer length
		int			BufLength;
		//! Byte length
		int			ByteLength;
		//! Ordinal position
		int			OrdPos;
		//! Default value
		VDKString	Default;
		//! Is nullable
		VDKString	IsNullable;
	public:
		/*!
		Constructor
		*/
		VDKODBCColumn() {
			DataType	= 0;
			DecDigits	= 0;
			Radix		= 0;
			Nullable	= 0;
			SqlDataType	= 0;
			DateTimeSub	= 0;
			Size		= 0;
			BufLength	= 0;
			ByteLength	= 0;
			OrdPos		= 0;
		};
		/*!
		Destructor
		*/
		~VDKODBCColumn() {};
		/*!
		Returns SQL data type
		This can be an ODBC SQL data type or a driver-specific SQL data type.
		For datetime and interval data types,
		this column returns the concise data type
		(such as SQL_TYPE_DATE or SQL_INTERVAL_YEAR_TO_MONTH,
		rather than the nonconcise data type such as SQL_DATETIME or SQL_INTERVAL).
		*/
		int GetDataType() {
			return DataType;
		};
		/*!
		Returns the total number of significant digits to the right of the decimal point.
		For SQL_TYPE_TIME and SQL_TYPE_TIMESTAMP, this column contains the number
		of digits in the fractional seconds component.
		For the other data types, this is the decimal digits of the column
		on the data source.
		For interval data types that contain a time component,
		this column contains the number of digits to the right of the decimal point
		(fractional seconds).
		For interval data types that do not contain a time component, this column is 0.
		0 is returned for data types where GetDecDigits() is not applicable.
		*/
		int GetDecDigits() {
			return DecDigits;
		};
		/*!
		Returns radix.
		For numeric data types, either 10 or 2.
		If it is 10, the values in GetColSize() and GetDecDigits() give
		the number of decimal digits allowed for the column.
		For example, a DECIMAL(12,5) column would return a GetRadix() of 10,
		a GetColumnSize() of 12, and a GetDecDigits() of 5;
		a FLOAT column could return a GetRadix() of 10, a GetColumnSize() of 15,
		and a GetDecDigits() of 0.
		*/
		int GetRadix() {
			return Radix;
		};
		/*!
		Returns column nullablity. \n
		- SQL_NO_NULLS if the column could not include NULL values.
		- SQL_NULLABLE if the column accepts NULL values.
		- SQL_NULLABLE_UNKNOWN if it is not known whether the column accepts NULL values.
		*/
		int GetNullable() {
			return Nullable;
		};
		/*!
		Returns the default value of the column.
		The value in this column should be interpreted as a string
		if it is enclosed in quotation marks. 
		If NULL was specified as the default value,
		then this column is the word NULL, not enclosed in quotation marks.
		If the default value cannot be represented without truncation,
		then this column contains TRUNCATED, with no enclosing single quotation marks.
		If no default value was specified, then this column is NULL.
		The value of GetColDefault() can be used in generating a new column definition,
		except when it contains the value TRUNCATED. \n
		*/
		VDKString GetDefault() {
			return Default;
		};
		/*!
		Returns the SQL type.
		This can be an ODBC SQL data type or a driver-specific SQL data type.
		This column is the same as the GetDataType() column,
		with the exception of datetime and interval data types.
		This column returns the nonconcise data type (such as SQL_DATETIME or SQL_INTERVAL),
		rather than the concise data type (such as SQL_TYPE_DATE or SQL_INTERVAL_YEAR_TO_MONTH)
		for datetime and interval data types.
		If this column returns SQL_DATETIME or SQL_INTERVAL,
		the specific data type can be determined from the GetDateTimeSub() column. \n
		*/
		int GetSQLDataType() {
			return SqlDataType;
		};
		/*!
		Returns DateTime sub type
		The subtype code for datetime and interval data types.
		For other data types, this column returns a 0.
		*/
		int GetDateTimeSub() {
			return DateTimeSub;
		};
		/*!
		Returns the column nullability in string format.
		"NO" if the column does not include NULLs.
		"YES" if the column could include NULLs.
		This column returns a zero-length string if nullability is unknown.
		ISO rules are followed to determine nullability.
		An ISO SQL-compliant DBMS cannot return an empty string. 
		The value returned for this column is different from the value returned
		for the NULLABLE column. (See the description of the GetNullable() column.)
		*/
		VDKString GetIsNullable() {
			return IsNullable;
		};
		/*!
		Returns column size
		If GetDataType() is SQL_CHAR or SQL_VARCHAR,
		this column contains the maximum length in characters of the column.
		For datetime data types, this is the total number of characters required
		to display the value when converted to characters.
		For numeric data types, this is either the total number of digits or
		the total number of bits allowed in the column,
		according to the GetPrecision() column.
		For interval data types, this is the number of characters in the character
		representation of the interval literal
		(as defined by the interval leading precision).
		*/
		int GetSize() {
			return Size;
		};
		/*!
		Returns buffer length
		The field length in bytes of data for character data types,
		For numeric data, this size may be different than the size of
		the data stored on the data source.
		This value might be different than GetSize() column for character data.
		*/
		int GetBufLength() {
			return BufLength;
		};
		/*!
		Returns byte length
		The maximum length in bytes of a character or binary data type column.
		For all other data types, this column returns 0.
		*/
		int GetByteLength() {
			return ByteLength;
		};
		/*!
		Returns column ordinal position
		The ordinal position of the column in the table.
		The first column in the table is number 1.
		*/
		int GetOrdinalPos() {
			return OrdPos;
		};
		/*!
		Set int type column attribute value
		\param index Attribute type to set
		\param value New value
		Possible values for index :
		- VDKODBC_COL_TYPE			column type
		- VDKODBC_COL_DECDIGITS		column decimal digits
		- VDKODBC_COL_RADIX			column radix
		- VDKODBC_COL_NULLABLE		column nullable
		- VDKODBC_COL_SQLTYPE		column sql type
		- VDKODBC_COL_SQLDTSUB		column datetime sub
		- VDKODBC_COL_SIZE			column size
		- VDKODBC_COL_BUFLEN		column buffer length
		- VDKODBC_COL_BYTELEN		column byte length
		- VDKODBC_COL_ORDPOS		column ordinal position
		For internal use only.
		*/
		void SetIntValue(int index, int value);
		/*!
		Set string type column attribute value
		\param index Attribute type to set
		\param value New value
		Possible values for index :
		- VDKODBC_NAME				object name
		- VDKODBC_TYPENAME			object type name
		- VDKODBC_REMARKS			remarks
		- VDKODBC_COL_DEF			column default value
		- VDKODBC_COL_ISNULLABLE	column is nullable value
		For internal use only.
		*/
		virtual void SetStrValue(int index, const char *value);
};


#endif
