/*
 * ===========================
 * VDK Visual Development Kit
 * odbc subsystem
 * Version 0.1
 * Revision 0.0
 * January 2001
 * ===========================
 *
 * Copyright (C) 2001, Mario Motta
 * Developed by Pierre-Louis Malatray <pierrelouis.malatray@free.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#ifndef _vdkodbcconnect_h
#define _vdkodbcconnect_h

#include <vdk/dlist.h>
#include <vdkodbc/vdkodbc.h>
#include <vdkodbc/vdkodbctable.h>

//! ODBC resultset positions for SQLTables
#define VDKODBC_SQLTABLES_CATALOG		1
#define VDKODBC_SQLTABLES_SCHEMA		2
#define VDKODBC_SQLTABLES_NAME			3
#define VDKODBC_SQLTABLES_TYPE			4
#define VDKODBC_SQLTABLES_REMARKS		5

typedef VDKList<VDKODBCTable>			TableList;
typedef VDKListIterator<VDKODBCTable>	TableListIterator;

/*!
	\class VDKODBCConnection
	\brief Provides a connection to a configured unixODBC data source
*/
class VDKODBCConnection : public VDKODBCObject
{
	private:
		//! ODBC environment handle
		SQLHENV			HEnv;
		//! Connection handle
		SQLHDBC			HConn;
		//! Connection state indicator
		bool			connected;
		//! Data source name
		VDKString		DataSource;
		//! Completed connection string
		VDKString		ConnectionString;
		//! TimeOut setting
		unsigned int	TimeOut;
		//! Contains all table descriptors
		TableList		Tables;
	protected:
		/*!
		Initialize ODBC environment
		*/
		int AllocEnv();
		/*!
		Initialize ODBC connection
		*/
		int AllocConn();
		/*!
		Frees ODBC environment handle
		*/
		void FreeEnv();
		/*!
		Frees ODBC connection handle
		*/
		void FreeConn();
		/*!
		Prepares connection with allocating environment and connection
		handles and setting connection attributes
		*/
		int PrepareConnection();
		/*!
		Frees ODBC connection and environment handles
		*/
		void FreeConnection();
		/*!
		Set connected to newvalue
		*/
		void SetConnected(bool newvalue);
		/*!
		Build a dialog from the values in InStr.
		The user-supplied values are returned in OutStr
		\param InStr string coming from a call to DriverConnect or BrowseConnect
		\param OutStr Instr completed with user answers
		*/
		int GetUserInput(SQLCHAR *InStr, SQLCHAR *OutStr);
		/*!
		Feels an existing functions array
		*/
		int GetFunctions();
		/*!
		Returns tablenum table pointer
		\param tablenum Table number ( >= 0 and < GetNbTables())
		\return NULL if table not found
		*/
		VDKODBCTable* GetPTable(unsigned int tablenum);
		/*!
		Returns tablename table pointer
		\param tablename Table name (case sensitive)
		\return NULL if table not found
		*/
		VDKODBCTable* GetPTable(VDKString tablename);
	public:
		//! Constructor
		VDKODBCConnection();
		//! Destructor
		~VDKODBCConnection();
		/*!
		Connect to the datasource
		\param Datasource Name of a configured ODBC datasource
		\param UserName Login for the datasource
		\param Pwd Password for the datasource
		*/
		int Connect(VDKString Datasource, VDKString UserName,
					VDKString Pwd);
		/*!
		Disconnect from the datasource
		*/
		int Disconnect();
		/*!
		Returns the connection state
		*/
		bool Connected() {
			return connected;
		};
		/*!
		Sets connection time out value
		\param timeout Connection time out in seconds
		*/
		void SetTimeOut(unsigned int timeout) {
			TimeOut = timeout;
		}
		/*!
		Returns connection time out value in seconds
		*/
		unsigned int GetTimeOut() {
			return TimeOut;
		};
		/*!
		Returns the connection handle
		*/
		SQLHDBC GetHdbc() {
			return HConn;
		};
		/*!
		Returns the environment handle
		*/
		SQLHENV GetHenv() {
			return HEnv;
		};
		/*!
		Returns the data source name
		\warning Returned string will contain a NULL buffer
		if Connected is false
		*/
		VDKString GetDataSourceName() {
			return DataSource;
		};
		/*!
		Returns the completed connection string
		after using DriverConnect or BrowseConnect \n
		The application can use this string for subsequent connection requests
		*/
		VDKString GetConnectString() {
			return ConnectionString;
		};
		/*!
		Returns data sources available on the local machine.
		The returned string is formatted as follows : \n
		DSN|Driver name, one line per data source where '|' is a tabulation.
		\param Result Output string containing data sources names
		\param user_or_system_or_both can take three values : \n
		- VDKODBC_FETCH_ALL (default), GetDataSources will return ALL DSNs. \n
		- VDKODBC_FETCH_USER, GetDataSources will return user DSNs. \n
		- VDKODBC_FETCH_SYSTEM, GetDataSources will return system DSNs.
		\warning In case of error, the last error string will be returned\n
		no call to GetLastError() is needed.
		*/
		int GetDataSources(VDKString& Result, int user_or_system_or_both = VDKODBC_FETCH_ALL);
		/*!
		Lists all configured drivers descriptions and drivers attributes keywords.
		The returned string is formatted as follows : \n
		Driver name|Driver attribute1|Driver attribute2|..., one line per driver,
		where '|' is a tabulation.
		\param Result Output string containing driver names
		\return SQL_SUCCESS on success or an error number otherwise, \n
		call GetLastError() to get the error definition string.
		*/
		int GetDrivers(VDKString& Result);
		/*!
		Fills tables list, stored in the connected data source,
		fills each table attributes and privileges.
		\return SQL_SUCCESS on success or an error number otherwise, \n
		call GetLastError() to get the error definition string.
		\warning An error occurs if not connected.
		*/
		int LoadTablesAttr();
		/*!
		Returns data source tables number
		\warning Returns 0 if not connected
		*/
		unsigned int GetNbTables();
		/*!
		Returns a table name
		\param tablenum Table number ( >= 0 and < GetNbTables())
		\warning May return isNull() string
		*/
		VDKString GetTableName(unsigned int tablenum);
		/*!
		Returns a table type
		\param tablenum Table number ( >= 0 and < GetNbTables())
		\warning May return isNull() string
		*/
		VDKString GetTableType(unsigned int tablenum);
		/*!
		Returns a table type
		\param tablename Table name
		\warning May return isNull() string
		*/
		VDKString GetTableType(VDKString tablename);
		/*!
		Returns a table remarks
		\param tablenum Table number ( >= 0 and < GetNbTables())
		\warning May return isNull() string
		*/
		VDKString GetTableRemarks(unsigned int tablenum);
		/*!
		Returns a table remarks
		\param tablename Table name
		\warning May return isNull() string
		*/
		VDKString GetTableRemarks(VDKString tablename);
		/*!
		Returns a table catalog
		\param tablenum Table number ( >= 0 and < GetNbTables())
		\warning May return isNull() string
		*/
		VDKString GetTableCatalog(unsigned int tablenum);
		/*!
		Returns a table catalog
		\param tablename Table name
		\warning May return isNull() string
		*/
		VDKString GetTableCatalog(VDKString tablename);
		/*!
		Returns a table schema
		\param tablenum Table number ( >= 0 and < GetNbTables())
		\warning May return isNull() string
		*/
		VDKString GetTableSchema(unsigned int tablenum);
		/*!
		Returns a table schema
		\param tablename Table name
		\warning May return isNull() string
		*/
		VDKString GetTableSchema(VDKString tablename);
		/*!
		Returns table number from table name or
		VDKODBC_ERR_TABLE_NOT_FOUND if table not found
		\param tablename Table name
		*/
		int GetTableNum(VDKString tablename);
		/*!
		Fills table columns list, stored in the connected data source,
		fills each column attributes and privileges.
		\return SQL_SUCCESS on success or an error number otherwise, \n
		call GetLastError() to get the error definition string.
		\warning An error occurs if not connected.
		*/
		int LoadColumnsAttr(VDKString TableName);
		/*!
		Returns column quantity for table tablename
		\param tablename Table name
		*/
		unsigned int GetNbColumns(VDKString tablename);
		/*!
		Returns column name
		\param tablename Table name
		\param colnum Column number ( >= 0 and < GetNbColumns())
		\warning May return isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetColumnName(VDKString tablename, unsigned int colnum);
		/*!
		Returns column type name
		\param tablename Table name
		\param colname Column name
		\warning May return isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetColumnTypeName(VDKString tablename, VDKString colname);
		/*!
		Returns column remarks
		\param tablename Table name
		\param colname Column name
		\warning May return isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetColumnRemarks(VDKString tablename, VDKString colname);
		/*!
		Returns column size or
		VDKODBC_ERR_TABLE_NOT_FOUND if table not found or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param tablename Table name
		\param colname Column name
		\return Column size on success or an error number otherwise, \n
		call GetLastError() to get the error definition string.
		*/
		int GetColumnSize(VDKString tablename, VDKString colname);
		/*!
		Returns column buffer length or
		VDKODBC_ERR_TABLE_NOT_FOUND if table not found or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param tablename Table name
		\param colname Column name
		\return Buffer length on success or an error number otherwise, \n
		call GetLastError() to get the error definition string.
		*/
		int GetColumnBufLength(VDKString tablename, VDKString colname);
		/*!
		Returns column decimal digits or
		VDKODBC_ERR_TABLE_NOT_FOUND if table not found or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param tablename Table name
		\param colname Column name
		\return Decimal digits on success or an error number otherwise, \n
		call GetLastError() to get the error definition string.
		*/
		int GetColumnDecDigits(VDKString tablename, VDKString colname);
		/*!
		Returns column radix or
		VDKODBC_ERR_TABLE_NOT_FOUND if table not found or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param tablename Table name
		\param colname Column name
		\return Column radix on success or an error number otherwise, \n
		call GetLastError() to get the error definition string.
		*/
		int GetColumnRadix(VDKString tablename, VDKString colname);
		/*!
		Returns column default value
		\param tablename Table name
		\param colname Column name
		\warning May return isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetColumnDefaultValue(VDKString tablename, VDKString colname);
		/*!
		Returns column datetime sub or
		VDKODBC_ERR_TABLE_NOT_FOUND if table not found or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param tablename Table name
		\param colname Column name
		\return Column Date Time Sub on success or an error number otherwise, \n
		call GetLastError() to get the error definition string.
		*/
		int GetColumnDateTimeSub(VDKString tablename, VDKString colname);
		/*!
		Returns column ordinal position or
		VDKODBC_ERR_TABLE_NOT_FOUND if table not found or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param tablename Table name
		\param colname Column name
		\return Column Ordinal Pos on success or an error number otherwise, \n
		call GetLastError() to get the error definition string.
		*/
		int GetColumnOrdinalPos(VDKString tablename, VDKString colname);
		/*!
		Returns column nullability or
		VDKODBC_ERR_TABLE_NOT_FOUND if table not found or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param tablename Table name
		\param colname Column name
		\return Column Nullable on success or an error number otherwise, \n
		call GetLastError() to get the error definition string.
		*/
		int GetColumnNullable(VDKString tablename, VDKString colname);
		/*!
		Returns column nullability string
		\param tablename Table name
		\param colname Column name
		\warning May return isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetColumnIsNullable(VDKString tablename, VDKString colname);
		/*!
		Fills table privileges list, stored in the connected data source,
		\return SQL_SUCCESS on success or an error number otherwise, \n
		call GetLastError() to get the error definition string.
		\warning An error occurs if not connected.
		*/
		int LoadTablePrivileges(VDKString TableName);

//================================================================================================================
//  Ci-dessous les mthodes  revoir et  complter
//================================================================================================================
		/*
		DriverConnect is an alternative to Connect. \n
		It supports data sources that require more connection information than the three arguments in Connect,
		dialog boxes to prompt the user for all connection information,
		and data sources that are not defined in the system information
		\param ConnectionString Formatted string to connect
		\param WindowHandle Handle of owner window
		\param Completion Flag to indicate the driver how to treat this connection request \n
		possible values : SQL_DRIVER_PROMPT, SQL_DRIVER_COMPLETE, SQL_DRIVER_COMPLETE_REQUIRED, or SQL_DRIVER_NOPROMPT
		*/
		int DriverConnect(VDKString ConnectionString, void *WindowHandle = NULL,
						  unsigned short Completion = SQL_DRIVER_PROMPT);
		/*
		DriverConnect is an alternative to Connect. \n
		It supports data sources that require more connection information than the three arguments in Connect,
		dialog boxes to prompt the user for all connection information,
		and data sources that are not defined in the system information.
		DriverConnect provides the following connection attributes: 
		- Establish a connection using a connection string that contains the data source name,
		one or more user IDs, one or more passwords, and other information required by the data source. 
		- Establish a connection using a partial connection string or no additional information;
		in this case, the Driver Manager and the driver can each prompt the user for connection information. 
		- Establish a connection to a data source that is not defined in the system information.
		If the application supplies a partial connection string, the driver can prompt the user for connection information. 
		- Establish a connection to a data source using a connection string constructed from the information in a .dsn file. \n
		After a connection is established, SQLDriverConnect returns the completed connection string. \n
		The application can use this string for subsequent connection requests. \n
		For more details about Connection string format, see the URL provided on this documentation mainpage.
		\param ConnectionString Initial connection string
		*/
		int BrowseConnect(VDKString ConnectionString);
		/*
		*/
		int GetInfo(unsigned short infotype, void *answer, short bufferlen);
		/*
		Indicate if the specified driver function exists or not
		\param funcID Function identifier as defined in sql.h
		*/
		bool FuncExists(unsigned short funcID);
		/*
		*/
		VDKString GetDrvFunc();
};

#endif

